h1 {
  font-size:2em;
}

h2 {
  font-size:1.3em;
}

/* don't indent first paragraph */
div + p, h1 + p, h2 + p, h3 + p,
li p:first-child,
p.dropcap,
div.note p:nth-child(2), div.tip p:nth-child(2){
    text-indent: 0;
}

p {
    text-indent: .5em;
}

pre {
    margin-left: 2em;
    font-size: .5em;
}

div.note, div.tip {
    margin-left: 1em;
    /* width: 2em; */

}
  div.note p, div.tip p {
      font-style: italic;
      text-indent: 0em;
  }

  div.note .admonition-title, div.tip .admonition-title {
      font-variant: small-caps;
      text-indent: 0px;
  }
