html, body, div, span, applet, object, iframe, h1, h2, h3, h4, h5, h6,
p, blockquote, /* pre */, a, abbr, acronym, address, big, cite, code, del,
dfn, em, font, img, ins, kbd, q, s, samp, small, strike, strong, sub,
sup, tt, var, b, u, i, center, dl, dt, dd, ol, ul, li, fieldset, form,
label, legend, table, caption, tbody, tfoot, thead, tr, th, td {
  /* margin: 0; */
  padding: 0;
  border: 0;
  outline: 0;
  font-size: 100%;
  vertical-align: baseline;
  background: transparent; }

/* end reset */

@page {
    margin-top: 30px;
    margin-bottom: 30px;
}

body {
    padding: 0;
}

h1 {
  font-size: 1.5em;
}
h2 {
  font-size: 1.2em;
}
h3 {
  font-size: 1.1em;
}

h1, h2, h3 {
    page-break-after: avoid;
    margin-top:1em;
    margin-bottom:0;
}

p {
    line-height: 1.5em;
    text-indent: 1em;
    margin: 0;
    text-align: justify;
    orphans: 2;
    widow: 2;
}


h1>p:first-child {
}

/* don't indent first paragraph */
div + p, h1 + p, h2 + p, h3 + p,
li p:first-child,
p.dropcap,
div.note p:nth-child(2), div.tip p:nth-child(2){
    text-indent: 0;
}


pre {
    margin-left: 2em;
    font-size: .8em;
    white-space: pre-wrap;
    display: block;
}


ul, ol { margin-left: 1em; }
dd { margin: 0 0 0 40px; }
nav ul, nav ol { list-style: none; list-style-image: none; margin: 0; padding: 0; }

table {
  border:1px solid #ccc;
  border-collapse:collapse;
  width:90%;
  margin:5px auto 5px auto;}

thead, tfoot {
  background-color:#D0E3FA;
  border:1px solid #aaa;}

tbody {background-color:#FFFFFF;}

th {
  font-family:monospace;
  border:1px dotted #aaa;
  padding:5px;
  background-color:#EFF6FF;}

td {
  font-family:sans-serif;
  font-size:80%;
  border:1px solid #ccc;
  padding:5px;
  text-align:left;}

img {
    display: inline-block;
    width: 100%;
    margin: 1.5em;
    text-align: center;
}




div.note, div.tip {
    /* margin: 2em; */
    /* margin-bottom: 20px; */
    display: inline-block;
    /* border: 1px solid green; */
    font-size: 90%;
    width:90%;
}
  div.note p, div.tip p {
      font-style: italic;
      text-indent: 0px;
 /* margin: 2em; */
      margin-bottom: 1em;
      margin-left: 2em;
      margin-right: 2em;
  }

  div.note .admonition-title, div.tip .admonition-title {
      margin-top: 1em;
      /* border: 1px solid red; */
      font-style: oblique;
      font-variant: small-caps;
      text-transform: uppercase;
      text-align:center;
      page-break-after: avoid;
  }
