<html xmlns="http://www.w3.org/1999/xhtml"
    xmlns:py="http://genshi.edgewall.org/">
<head>
  <title>${book.title}</title>
  <style type="text/css">
.title, .authors {
  text-align: center;
}
span.author {
  margin: 1em;
}
  </style>
</head>
<body>
  <h1 class="title">${book.title}</h1>
  <h3 class="authors">
  <py:for each="creator, _ in book.creators">
  <span class="author">$creator</span>
  </py:for>
  </h3>
</body>
</html>
